@@ -54,18 +54,22 @@
     "split_data",
     "type",
     "x",
+    "xaxl",
     "xaxs",
     "xaxt",
     "xlabs",
     "xlim",
+    "xlim_user",
     "xlvls",
     "xmax",
     "xmin",
     "y",
+    "yaxl",
     "yaxs",
     "yaxt",
     "ylabs",
     "ylim",
+    "ylim_user",
     "ymax",
     "ymin"
   ))
